@@ -344,6 +344,8 @@ def csv_expand_toml_tasks(config_wr: ConfigWorkRequirement, csv_file: str) -> Di
         (config_wr.flatten_input_paths, FLATTEN_PATHS),
         (config_wr.input_files, INPUT_FILES),
         (config_wr.output_files, OUTPUT_FILES),
+        (config_wr.task_data, TASK_DATA),
+        (config_wr.task_data_file, TASK_DATA_FILE),
         (config_wr.task_name, TASK_NAME),
         (config_wr.task_type, TASK_TYPE),
         (config_wr.upload_files, UPLOAD_FILES),
