@@ -81,8 +81,8 @@ CONFIG_WR: ConfigWorkRequirement = load_config_work_requirement()
 
 
 ID = generate_id(CONFIG_COMMON.name_tag)
-TASK_BATCH_SIZE = 2000
-INPUT_FOLDER_NAME = None
+TASK_BATCH_SIZE = CONFIG_WR.task_batch_size
+INPUTS_FOLDER_NAME = None
 
 if ARGS_PARSER.dry_run:
     WR_SNAPSHOT = WorkRequirementSnapshot()
@@ -98,7 +98,7 @@ def main():
         return
 
     wr_data_file = (
-        CONFIG_WR.tasks_data_file
+        CONFIG_WR.wr_data_file
         if ARGS_PARSER.work_req_file is None
         else ARGS_PARSER.work_req_file
     )
@@ -112,10 +112,10 @@ def main():
         )
 
     if wr_data_file is None and csv_files is not None:
-        tasks_data = csv_expand_toml_tasks(CONFIG_WR, csv_files[0])
+        wr_data = csv_expand_toml_tasks(CONFIG_WR, csv_files[0])
         submit_work_requirement(
             directory_to_upload_from=CONFIG_FILE_DIR,
-            tasks_data=tasks_data,
+            wr_data=wr_data,
         )
 
     elif wr_data_file is not None:
@@ -124,32 +124,32 @@ def main():
         # JSON file
         if wr_data_file.lower().endswith("json"):
             if csv_files is not None:
-                tasks_data = load_json_file_with_csv_task_expansion(
+                wr_data = load_json_file_with_csv_task_expansion(
                     json_file=wr_data_file,
                     csv_files=csv_files,
                 )
             else:
-                tasks_data = load_json_file_with_mustache_substitutions(wr_data_file)
+                wr_data = load_json_file_with_mustache_substitutions(wr_data_file)
 
         # Jsonnet file
         elif wr_data_file.lower().endswith("jsonnet"):
             if csv_files is not None:
-                tasks_data = load_jsonnet_file_with_csv_task_expansion(
+                wr_data = load_jsonnet_file_with_csv_task_expansion(
                     jsonnet_file=wr_data_file,
                     csv_files=csv_files,
                 )
             else:
-                tasks_data = load_jsonnet_file_with_mustache_substitutions(wr_data_file)
+                wr_data = load_jsonnet_file_with_mustache_substitutions(wr_data_file)
 
         # TOML file (undocumented)
         elif wr_data_file.lower().endswith("toml"):
             if csv_files is not None:
-                tasks_data = load_toml_file_with_csv_task_expansion(
+                wr_data = load_toml_file_with_csv_task_expansion(
                     toml_file=wr_data_file,
                     csv_files=csv_files,
                 )
             else:
-                tasks_data = load_toml_file_with_mustache_substitutions(wr_data_file)
+                wr_data = load_toml_file_with_mustache_substitutions(wr_data_file)
 
         # None of the above
         else:
@@ -157,10 +157,10 @@ def main():
                 f"Work Requirement data file '{wr_data_file}' "
                 "must end with '.json', '.jsonnet', or '.toml'"
             )
-        validate_properties(tasks_data, "Work Requirement JSON")
+        validate_properties(wr_data, "Work Requirement JSON")
         submit_work_requirement(
             directory_to_upload_from=dirname(wr_data_file),
-            tasks_data=tasks_data,
+            wr_data=wr_data,
         )
 
     else:
@@ -179,7 +179,7 @@ def main():
 
 def submit_work_requirement(
     directory_to_upload_from: str,
-    tasks_data: Optional[Dict] = None,
+    wr_data: Optional[Dict] = None,
     task_count: Optional[int] = None,
 ):
     """
@@ -192,17 +192,17 @@ def submit_work_requirement(
     Task > Task Group > Top-Level JSON Property > TOML config file
     """
     # Create a default tasks_data dictionary if required
-    tasks_data = {TASK_GROUPS: [{TASKS: [{}]}]} if tasks_data is None else tasks_data
-    check_dict(tasks_data)
+    wr_data = {TASK_GROUPS: [{TASKS: [{}]}]} if wr_data is None else wr_data
+    check_dict(wr_data)
 
     # Remap 'task_type' at WR level to 'task_types' if 'task_types' is empty
-    if tasks_data.get(TASK_TYPE, None) is not None:
-        if tasks_data.get(TASK_TYPES, None) is None:
-            tasks_data[TASK_TYPES] = [tasks_data[TASK_TYPE]]
+    if wr_data.get(TASK_TYPE, None) is not None:
+        if wr_data.get(TASK_TYPES, None) is None:
+            wr_data[TASK_TYPES] = [wr_data[TASK_TYPE]]
 
     # Overwrite the WR name?
     global ID
-    ID = tasks_data.get(WR_NAME, ID if CONFIG_WR.wr_name is None else CONFIG_WR.wr_name)
+    ID = wr_data.get(WR_NAME, ID if CONFIG_WR.wr_name is None else CONFIG_WR.wr_name)
 
     # Handle any files that need to be uploaded
     global UPLOADED_FILES
@@ -214,18 +214,18 @@ def submit_work_requirement(
 
     # Flatten upload paths?
     flatten_upload_paths = check_bool(
-        tasks_data.get(FLATTEN_UPLOAD_PATHS, CONFIG_WR.flatten_upload_paths)
+        wr_data.get(FLATTEN_UPLOAD_PATHS, CONFIG_WR.flatten_upload_paths)
     )
 
     # Create the list of Task Groups
     task_groups: List[TaskGroup] = []
-    for tg_number, task_group_data in enumerate(tasks_data[TASK_GROUPS]):
-        task_groups.append(create_task_group(tg_number, tasks_data, task_group_data))
+    for tg_number, task_group_data in enumerate(wr_data[TASK_GROUPS]):
+        task_groups.append(create_task_group(tg_number, wr_data, task_group_data))
 
     # Create the Work Requirement
-    priority = check_float_or_int(tasks_data.get(PRIORITY, CONFIG_WR.priority))
+    priority = check_float_or_int(wr_data.get(PRIORITY, CONFIG_WR.priority))
     fulfilOnSubmit = check_bool(
-        tasks_data.get(FULFIL_ON_SUBMIT, CONFIG_WR.fulfil_on_submit)
+        wr_data.get(FULFIL_ON_SUBMIT, CONFIG_WR.fulfil_on_submit)
     )
     work_requirement = WorkRequirement(
         namespace=CONFIG_COMMON.namespace,
@@ -252,18 +252,15 @@ def submit_work_requirement(
             add_tasks_to_task_group(
                 tg_number,
                 task_group,
-                tasks_data,
+                wr_data,
                 task_count,
                 work_requirement,
                 flatten_upload_paths=flatten_upload_paths,
             )
 
         except Exception as e:
-            print_error(e)
             cleanup_on_failure(work_requirement)
-            if ARGS_PARSER.debug:
-                raise e
-            return
+            raise e
 
     if ARGS_PARSER.follow:
         follow_progress(work_requirement)
@@ -271,7 +268,7 @@ def submit_work_requirement(
 
 def create_task_group(
     tg_number: int,
-    tasks_data: Dict,
+    wr_data: Dict,
     task_group_data: Dict,
 ) -> TaskGroup:
     """
@@ -283,9 +280,9 @@ def create_task_group(
     if task_group_data.get(TASK_TYPE, None) is not None:
         if task_group_data.get(TASK_TYPES, None) is None:
             task_group_data[TASK_TYPES] = [task_group_data[TASK_TYPE]]
-    task_types_from_tasks = set()
 
     # Gather task types
+    task_types_from_tasks = set()
     for task in task_group_data[TASKS]:
         try:
             task_types_from_tasks.add(task[TASK_TYPE])
@@ -293,7 +290,7 @@ def create_task_group(
             pass
 
     # Name the Task Group
-    num_task_groups = len(tasks_data[TASK_GROUPS])
+    num_task_groups = len(wr_data[TASK_GROUPS])
     num_tasks = len(task_group_data[TASKS])
     task_group_name = get_task_group_name(
         task_group_data.get(NAME, CONFIG_WR.task_group_name),
@@ -307,7 +304,7 @@ def create_task_group(
     # specified in the Tasks.
     task_types: List = list(
         set(
-            check_list(task_group_data.get(TASK_TYPES, tasks_data.get(TASK_TYPES, [])))
+            check_list(task_group_data.get(TASK_TYPES, wr_data.get(TASK_TYPES, [])))
         ).union(task_types_from_tasks)
     )
     if CONFIG_WR.task_type is not None and CONFIG_WR.task_type not in task_types:
@@ -316,7 +313,7 @@ def create_task_group(
         raise Exception(f"No Task Type(s) specified in Task Group '{task_group_name}'")
 
     vcpus_data: Optional[List[float]] = check_list(
-        task_group_data.get(VCPUS, tasks_data.get(VCPUS, CONFIG_WR.vcpus))
+        task_group_data.get(VCPUS, wr_data.get(VCPUS, CONFIG_WR.vcpus))
     )
     vcpus = (
         None
@@ -325,7 +322,7 @@ def create_task_group(
     )
 
     ram_data: Optional[List[float]] = check_list(
-        task_group_data.get(RAM, tasks_data.get(RAM, CONFIG_WR.ram))
+        task_group_data.get(RAM, wr_data.get(RAM, CONFIG_WR.ram))
     )
     ram = (
         None
@@ -334,7 +331,7 @@ def create_task_group(
     )
 
     providers_data: Optional[List[str]] = check_list(
-        task_group_data.get(PROVIDERS, tasks_data.get(PROVIDERS, CONFIG_WR.providers))
+        task_group_data.get(PROVIDERS, wr_data.get(PROVIDERS, CONFIG_WR.providers))
     )
     providers: Optional[List[CloudProvider]] = (
         None
@@ -346,23 +343,23 @@ def create_task_group(
         taskTypes=task_types,
         maximumTaskRetries=check_int(
             task_group_data.get(
-                MAX_RETRIES, tasks_data.get(MAX_RETRIES, CONFIG_WR.max_retries)
+                MAX_RETRIES, wr_data.get(MAX_RETRIES, CONFIG_WR.max_retries)
             )
         ),
         workerTags=check_list(
             task_group_data.get(
-                WORKER_TAGS, tasks_data.get(WORKER_TAGS, CONFIG_WR.worker_tags)
+                WORKER_TAGS, wr_data.get(WORKER_TAGS, CONFIG_WR.worker_tags)
             )
         ),
         exclusiveWorkers=check_bool(
             task_group_data.get(
                 EXCLUSIVE_WORKERS,
-                tasks_data.get(EXCLUSIVE_WORKERS, CONFIG_WR.exclusive_workers),
+                wr_data.get(EXCLUSIVE_WORKERS, CONFIG_WR.exclusive_workers),
             )
         ),
         instanceTypes=check_list(
             task_group_data.get(
-                INSTANCE_TYPES, tasks_data.get(INSTANCE_TYPES, CONFIG_WR.instance_types)
+                INSTANCE_TYPES, wr_data.get(INSTANCE_TYPES, CONFIG_WR.instance_types)
             )
         ),
         vcpus=vcpus,
@@ -374,13 +371,13 @@ def create_task_group(
         ),
         providers=providers,
         regions=check_list(
-            task_group_data.get(REGIONS, tasks_data.get(REGIONS, CONFIG_WR.regions))
+            task_group_data.get(REGIONS, wr_data.get(REGIONS, CONFIG_WR.regions))
         ),
     )
     ctttl_data = check_float_or_int(
         task_group_data.get(
             COMPLETED_TASK_TTL,
-            tasks_data.get(COMPLETED_TASK_TTL, CONFIG_WR.completed_task_ttl),
+            wr_data.get(COMPLETED_TASK_TTL, CONFIG_WR.completed_task_ttl),
         )
     )
     completed_task_ttl = None if ctttl_data is None else timedelta(minutes=ctttl_data)
@@ -393,7 +390,7 @@ def create_task_group(
         finishIfAllTasksFinished=check_bool(
             task_group_data.get(
                 FINISH_IF_ALL_TASKS_FINISHED,
-                tasks_data.get(
+                wr_data.get(
                     FINISH_IF_ALL_TASKS_FINISHED, CONFIG_WR.finish_if_all_tasks_finished
                 ),
             )
@@ -401,13 +398,13 @@ def create_task_group(
         finishIfAnyTaskFailed=check_bool(
             task_group_data.get(
                 FINISH_IF_ANY_TASK_FAILED,
-                tasks_data.get(
+                wr_data.get(
                     FINISH_IF_ANY_TASK_FAILED, CONFIG_WR.finish_if_any_task_failed
                 ),
             )
         ),
         priority=check_float_or_int(
-            task_group_data.get(PRIORITY, tasks_data.get(PRIORITY, CONFIG_WR.priority))
+            task_group_data.get(PRIORITY, wr_data.get(PRIORITY, CONFIG_WR.priority))
         ),
         completedTaskTtl=completed_task_ttl,
     )
@@ -419,7 +416,7 @@ def create_task_group(
 def add_tasks_to_task_group(
     tg_number: int,
     task_group: TaskGroup,
-    tasks_data: Dict,
+    wr_data: Dict,
     task_count: Optional[int],
     work_requirement: WorkRequirement,
     flatten_upload_paths: bool = False,
@@ -429,31 +426,34 @@ def add_tasks_to_task_group(
     """
 
     # Ensure there's at least one Task
-    num_tasks = len(tasks_data[TASK_GROUPS][tg_number][TASKS])
+    num_tasks = len(wr_data[TASK_GROUPS][tg_number][TASKS])
     if num_tasks == 0:
-        tasks_data[TASK_GROUPS][tg_number][TASKS] = [{}]
+        wr_data[TASK_GROUPS][tg_number][TASKS] = [{}]
         num_tasks = 1
 
-    # If 'taskCount' is set at the Task Group level, and there
-    # is only one Task, create 'taskCount' copies of the Task
-    task_group_task_count = tasks_data[TASK_GROUPS][tg_number].get(TASK_COUNT, None)
+    # If 'taskCount' is set at the Json Work Requirement or
+    # Task Group levels, and there is only one Task, create 'taskCount'
+    # copies of the Task. Note: NOT inherited from the TOML level.
+    task_group_task_count = wr_data[TASK_GROUPS][tg_number].get(
+        TASK_COUNT, wr_data.get(TASK_COUNT, None)
+    )
     if task_group_task_count is not None:
         if num_tasks == 1:
             task_count = check_int(task_group_task_count)
         else:
-            raise Exception(
-                "Can't use 'taskCount' for a Task Group if there are "
-                "multiple Tasks in the group"
+            print_log(
+                f"Warning: Task Group '{task_group.name}' contains {num_tasks} "
+                "Tasks: ignoring 'taskCount'"
             )
 
-    num_task_groups = len(tasks_data[TASK_GROUPS])
+    num_task_groups = len(wr_data[TASK_GROUPS])
 
     # Determine Task batching
-    tasks = tasks_data[TASK_GROUPS][tg_number][TASKS]
+    tasks = wr_data[TASK_GROUPS][tg_number][TASKS]
     num_tasks = len(tasks) if task_count is None else task_count
     num_task_batches: int = ceil(num_tasks / TASK_BATCH_SIZE)
     tasks_list: List[Task] = []
-    if num_task_batches > 1:
+    if num_task_batches > 1 and not ARGS_PARSER.dry_run:
         print_log(
             f"Adding Tasks to Task Group '{task_group.name}' in "
             f"{num_task_batches} batches"
@@ -467,7 +467,7 @@ def add_tasks_to_task_group(
             (TASK_BATCH_SIZE * batch_number),
             min(TASK_BATCH_SIZE * (batch_number + 1), num_tasks),
         ):
-            task_group_data = tasks_data[TASK_GROUPS][tg_number]
+            task_group_data = wr_data[TASK_GROUPS][tg_number]
             task = tasks[task_number] if task_count is None else tasks[0]
             task_name = get_task_name(
                 task.get(NAME, CONFIG_WR.task_name),
@@ -480,20 +480,18 @@ def add_tasks_to_task_group(
                 task.get(
                     EXECUTABLE,
                     task_group_data.get(
-                        EXECUTABLE, tasks_data.get(EXECUTABLE, CONFIG_WR.executable)
+                        EXECUTABLE, wr_data.get(EXECUTABLE, CONFIG_WR.executable)
                     ),
                 )
             )
             arguments_list = check_list(
                 task.get(
                     ARGS,
-                    tasks_data.get(ARGS, task_group_data.get(ARGS, CONFIG_WR.args)),
+                    wr_data.get(ARGS, task_group_data.get(ARGS, CONFIG_WR.args)),
                 )
             )
             env = check_dict(
-                task.get(
-                    ENV, task_group_data.get(ENV, tasks_data.get(ENV, CONFIG_WR.env))
-                )
+                task.get(ENV, task_group_data.get(ENV, wr_data.get(ENV, CONFIG_WR.env)))
             )
 
             # Set up lists of files to input, verify
@@ -502,7 +500,7 @@ def add_tasks_to_task_group(
                     INPUT_FILES,
                     task_group_data.get(
                         INPUT_FILES,
-                        tasks_data.get(INPUT_FILES, CONFIG_WR.input_files),
+                        wr_data.get(INPUT_FILES, CONFIG_WR.input_files),
                     ),
                 )
             )
@@ -511,7 +509,7 @@ def add_tasks_to_task_group(
                     VERIFY_AT_START,
                     task_group_data.get(
                         VERIFY_AT_START,
-                        tasks_data.get(VERIFY_AT_START, CONFIG_WR.verify_at_start),
+                        wr_data.get(VERIFY_AT_START, CONFIG_WR.verify_at_start),
                     ),
                 )
             )
@@ -519,7 +517,7 @@ def add_tasks_to_task_group(
                 task.get(
                     VERIFY_WAIT,
                     task_group_data.get(
-                        VERIFY_WAIT, tasks_data.get(VERIFY_WAIT, CONFIG_WR.verify_wait)
+                        VERIFY_WAIT, wr_data.get(VERIFY_WAIT, CONFIG_WR.verify_wait)
                     ),
                 )
             )
@@ -541,7 +539,7 @@ def add_tasks_to_task_group(
                     UPLOAD_FILES,
                     task_group_data.get(
                         UPLOAD_FILES,
-                        tasks_data.get(UPLOAD_FILES, CONFIG_WR.upload_files),
+                        wr_data.get(UPLOAD_FILES, CONFIG_WR.upload_files),
                     ),
                 )
             )
@@ -558,7 +556,7 @@ def add_tasks_to_task_group(
             inputs = generate_task_input_list(
                 files=[
                     unique_upload_pathname(
-                        input_file, ID, INPUT_FOLDER_NAME, False, flatten_upload_paths
+                        input_file, ID, INPUTS_FOLDER_NAME, False, flatten_upload_paths
                     )
                     for input_file in input_files_list
                 ],
@@ -580,7 +578,7 @@ def add_tasks_to_task_group(
                         OUTPUT_FILES,
                         task_group_data.get(
                             OUTPUT_FILES,
-                            tasks_data.get(OUTPUT_FILES, CONFIG_WR.output_files),
+                            wr_data.get(OUTPUT_FILES, CONFIG_WR.output_files),
                         ),
                     )
                 )
@@ -592,9 +590,7 @@ def add_tasks_to_task_group(
                     CAPTURE_TASKOUTPUT,
                     task_group_data.get(
                         CAPTURE_TASKOUTPUT,
-                        tasks_data.get(
-                            CAPTURE_TASKOUTPUT, CONFIG_WR.capture_taskoutput
-                        ),
+                        wr_data.get(CAPTURE_TASKOUTPUT, CONFIG_WR.capture_taskoutput),
                     ),
                 )
             ):
@@ -613,13 +609,16 @@ def add_tasks_to_task_group(
 
             tasks_list.append(
                 create_task(
-                    tasks_data=tasks_data,
+                    wr_data=wr_data,
                     task_group_data=task_group_data,
                     task_data=task,
                     name=task_name,
                     task_type=task_type,
                     executable=executable,
                     args=arguments_list,
+                    task_data_property=get_task_data_property(
+                        wr_data, task_group_data, task, task_name
+                    ),
                     env=env,
                     inputs=inputs,
                     outputs=outputs,
@@ -655,6 +654,36 @@ def add_tasks_to_task_group(
         )
 
 
+def get_task_data_property(
+    wr_data: Dict, task_group_data: Dict, task: Dict, task_name: str
+) -> Optional[str]:
+    """
+    Get the 'taskData' property, either using the contents of the file
+    specified in 'taskDataFile' or using the string specified in 'taskData'.
+    Raise exception if both 'taskData' and 'taskDataFile' are set at the same
+    level in the Work Requirement.
+    """
+
+    # Try Task, Task Group, then Work Requirement data
+    for data, task_data_default, task_data_file_default in [
+        (task, None, None),
+        (task_group_data, None, None),
+        (wr_data, CONFIG_WR.task_data, CONFIG_WR.task_data_file),
+    ]:
+        task_data_property = data.get(TASK_DATA, task_data_default)
+        task_data_file_property = data.get(TASK_DATA_FILE, task_data_file_default)
+        if task_data_property and task_data_file_property:
+            raise Exception(
+                f"Task '{task_name}': Properties '{TASK_DATA}' and "
+                f"'{TASK_DATA_FILE}' are both set"
+            )
+        if task_data_property:
+            return task_data_property
+        if task_data_file_property:
+            with open(task_data_file_property, "r") as f:
+                return f.read()
+
+
 def follow_progress(work_requirement: WorkRequirement) -> None:
     """
     Follow and report the progress of a Work Requirement
@@ -696,7 +725,7 @@ def cleanup_on_failure(work_requirement: WorkRequirement) -> None:
     print_log(f"Cancelled Work Requirement '{work_requirement.name}'")
 
     # Delete uploaded objects
-    UPLOADED_FILES.delete_uploaded_files()
+    UPLOADED_FILES.delete()
 
 
 def check_for_duplicates_in_file_lists(
@@ -804,23 +833,42 @@ def get_task_group_name(
 
 
 def create_task(
-    tasks_data: Dict,
+    wr_data: Dict,
     task_group_data: Dict,
     task_data: Dict,
     name: str,
     task_type: str,
     executable: str,
     args: List[str],
+    task_data_property: Optional[str],
     env: Dict[str, str],
     inputs: Optional[List[TaskInput]],
     outputs: Optional[List[TaskOutput]],
     flatten_upload_paths: bool = False,
 ) -> Task:
     """
-    Create a Task object, handling variations for different Task Types.
-    This is where to define a new Task Type and to set up how it's run.
+    Create a Task object, handling special processing for specific Task Types.
     """
 
+    def _make_task(flatten_input_paths: FlattenPath) -> Task:
+        """
+        Helper function to create the Task object.
+        """
+        # Cannot use flatten_input_paths if there are no inputs
+        if inputs is None or len(inputs) == 0:
+            flatten_input_paths = None
+
+        return Task(
+            name=name,
+            taskType=task_type,
+            arguments=args,
+            inputs=inputs,
+            environment=env,
+            outputs=outputs,
+            flattenInputPaths=flatten_input_paths,
+            taskData=task_data_property,
+        )
+
     check_list(args)
     check_dict(env)
 
@@ -831,17 +879,20 @@ def create_task(
             FLATTEN_PATHS,
             task_group_data.get(
                 FLATTEN_PATHS,
-                tasks_data.get(FLATTEN_PATHS, CONFIG_WR.flatten_input_paths),
+                wr_data.get(FLATTEN_PATHS, CONFIG_WR.flatten_input_paths),
             ),
         )
     ):
         flatten_input_paths = FlattenPath.FILE_NAME_ONLY
 
-    # Special processing for Bash tasks. The Bash script is uploaded if this
-    # hasn't already been done, and it's added to the list of required files.
+    # Special processing for Bash tasks if the 'executable' property is set
+    # The Bash script is uploaded if this hasn't already been done, and
+    # added to the list of required files.
     if task_type == "bash":
         if executable is None:
-            raise Exception("No 'executable' specified for 'bash' Task Type")
+            print_log("Note: no 'executable' specified for 'bash' Task Type")
+            return _make_task(flatten_input_paths)
+
         UPLOADED_FILES.add_input_file(
             filename=executable,
             flatten_upload_paths=flatten_upload_paths,
@@ -850,7 +901,7 @@ def create_task(
             unique_upload_pathname(
                 filename=executable,
                 id=ID,
-                input_folder_name=INPUT_FOLDER_NAME,
+                inputs_folder_name=INPUTS_FOLDER_NAME,
                 flatten_upload_paths=flatten_upload_paths,
             ),
             verification=TaskInputVerification.VERIFY_AT_START,
@@ -862,33 +913,38 @@ def create_task(
             unique_upload_pathname(
                 filename=executable,
                 id=ID,
-                input_folder_name=INPUT_FOLDER_NAME,
+                inputs_folder_name=INPUTS_FOLDER_NAME,
                 flatten_upload_paths=flatten_upload_paths,
             )
             if flatten_input_paths is None
             else basename(executable)
         ] + args
+        return _make_task(flatten_input_paths)
 
-    # Special processing for Docker tasks. Sets up the '-e' environment strings
-    # and the DockerHub username and password if specified.
+    # Special processing for Docker tasks if the 'executable property is set.
+    # Sets up the '-e' environment strings and the DockerHub username and
+    # password if specified.
     elif task_type == "docker":
         if executable is None:
-            raise Exception("No 'executable' specified for 'Docker' Task Type")
+            print_log("Note: no 'executable' specified for 'docker' Task Type")
+            return _make_task(flatten_input_paths)
+
         # Set up the environment variables to be sent to the Docker container
         docker_env = check_dict(
             task_data.get(
                 DOCKER_ENV,
                 task_group_data.get(
                     DOCKER_ENV,
-                    tasks_data.get(DOCKER_ENV, CONFIG_WR.docker_env),
+                    wr_data.get(DOCKER_ENV, CONFIG_WR.docker_env),
                 ),
             )
         )
-        docker_env_string = ""
+        # 'TASK_NAME' env. var. is set for convenience
+        docker_env_string = f"--env TASK_NAME={name}"
         if docker_env is not None:
             for key, value in docker_env.items():
                 docker_env_string += f" --env {key}={value}"
-            docker_env_string += f" --env TASK_NAME={name.replace(' ', '_')}"
+
         args = [docker_env_string, executable] + args
 
         # Set up the environment used by the script to run Docker
@@ -897,14 +953,14 @@ def create_task(
             DOCKER_USERNAME,
             task_group_data.get(
                 DOCKER_USERNAME,
-                tasks_data.get(DOCKER_USERNAME, CONFIG_WR.docker_username),
+                wr_data.get(DOCKER_USERNAME, CONFIG_WR.docker_username),
             ),
         )
         docker_password = task_data.get(
             DOCKER_PASSWORD,
             task_group_data.get(
                 DOCKER_PASSWORD,
-                tasks_data.get(DOCKER_PASSWORD, CONFIG_WR.docker_password),
+                wr_data.get(DOCKER_PASSWORD, CONFIG_WR.docker_password),
             ),
         )
         env.update(
@@ -915,25 +971,12 @@ def create_task(
             if docker_username is not None and docker_password is not None
             else {}
         )
+        return _make_task(flatten_input_paths)
 
     else:
         # All other Task Types are sent through without additional processing
         # of the uploaded files, arguments or environment.
-        pass
-
-    # Cannot use flatten_input_paths if there are no inputs
-    if inputs is None or len(inputs) == 0:
-        flatten_input_paths = None
-
-    return Task(
-        name=name,
-        taskType=task_type,
-        arguments=args,
-        inputs=inputs,
-        environment=env,
-        outputs=outputs,
-        flattenInputPaths=flatten_input_paths,
-    )
+        return _make_task(flatten_input_paths)
 
 
 def submit_json_raw(wr_file: str):
